(function() {

	// Store image data
	let imgArr = [
		{ src: 'images/photos/lounge2.jpg', alt: 'Lounge 2' },
		{ src: 'images/photos/mountainview.jpg', alt: 'Mountain View Office' },
		{ src: 'images/photos/air.jpg', alt: 'Googlair' },
		{ src: 'images/photos/racecar.jpg', alt: 'Racecar' },
		{ src: 'images/photos/desks.jpg', alt: 'Desks' },
		{ src: 'images/photos/lounge.jpg', alt: 'Lounge' },
		{ src: 'images/photos/hive.jpg', alt: 'Hive' },
		{ src: 'images/photos/logo.jpg', alt: 'Logo' }
	];

	// Store nodes in memory, used by getElement and getElements
	let foundElements = {};

	// Wrapper function to save node into memory (single selector)
	function getElement(querySelector) {
		if(foundElements[querySelector]) {
			return foundElements[querySelector];
		}
		else {
			foundElements[querySelector] = document.querySelector(querySelector);
			return foundElements[querySelector];
		}
	}

	// Wrapper function to save node into memory (multiple selector)
	function getElements(querySelector) {
		if(foundElements[querySelector]) {
			return foundElements[querySelector];
		}
		else {
			foundElements[querySelector] = document.querySelectorAll(querySelector);
			return foundElements[querySelector];
		}
	}

	/*
	 * Creates an image photo grid given an array of data
	 * Initializes a new carousel within the class
	 * Contains method to initiate header resize on scroll
	 */ 
	class ImgGrid {	
		constructor(images) {
			this.imgArr = images;
			this.init();
			const carousel = new Carousel(this.imgArr);
			carousel.init();
		}

		init() {
			// Create grid layout
			this.populateGrid();

			// Change header on scroll event
			this.resizeHeaderOnScroll();

			// Listen for tab key event, signalling accessibility
			window.addEventListener('keydown', this.handleFirstTabAccessibility);
		}

		// Populate thumbnail grid view
		populateGrid() {
			const galleryContainer = getElement('.gallery-container');

			for(let i = 0; i < imgArr.length; i++) {
				const photoContainer = document.createElement('div');
				photoContainer.classList.add('photo-container');
				
				const photo = document.createElement('div');
				photo.classList.add('photo');

				const photoImg = document.createElement('img');
				photoImg.classList.add('photo-img');
				photoImg.src = imgArr[i].src;
				photoImg.alt = imgArr[i].alt;
				photoImg.tabIndex = '0'; // Allow element to receive keyboard focus for accessibility

				galleryContainer.appendChild(photoContainer);
				photoContainer.appendChild(photo);
				photo.appendChild(photoImg);	
			}
		}

		// On scroll event, trigger header to be smaller or larger version
		resizeHeaderOnScroll() {
			const logoIcon = getElement('.logo-icon');
			const logoText = getElement('.logo-text');
			const mainHeader = getElement('.main-header');

			window.addEventListener('scroll', () => {
				if(document.documentElement.scrollTop > 5 && !mainHeader.classList.contains('header-scroll')) {
					mainHeader.classList.add('header-scroll');
					logoIcon.classList.add('logo-icon-scroll');
					logoText.classList.add('logo-text-scroll');
				}
				if(document.documentElement.scrollTop <= 5 && mainHeader.classList.contains('header-scroll')) {
					mainHeader.classList.remove('header-scroll');
					logoIcon.classList.remove('logo-icon-scroll');
					logoText.classList.remove('logo-text-scroll');
				}
			});
		}

		// Add accessibility class to body to handle focus styling if user pressed Tab
		handleFirstTabAccessibility(e) {
			const body = getElement('body');

			if(e.key === 'Tab') { // If first key entered is Tab, user relies on keyboard
				body.classList.add('accessibility');
				window.removeEventListener('keydown', this.handleFirstTabAccessibility);
			}
		}
	}

	/*
	 * Contains methods to add functionality to the carousel:
	 * activating thumbnails, nav buttons, overlay, keyboard, and swipe
	 */
	class Carousel {
		constructor(images) {
			this.imgArr = images; // Pass down imgArr data
			this.currIndex = 0; // Index of carousel image used in setActiveImg
		}

		init() {
			this.attachThumbnailListeners();
			this.attachNavListeners();
			this.attachOverlayListeners();
			this.attachKeyboardListeners();
			this.attachTouchListeners();
		}

		// When thumbnail is active using click or Enter, populate carousel overlay
		attachThumbnailListeners() {
			const thumbnails = getElements('.photo-img');
			
			for(let i = 0; i < thumbnails.length; i++) {
				thumbnails[i].addEventListener('click', () => {
					this.populateCarousel(i);
				});

				thumbnails[i].addEventListener('keydown', (e) => {
					if(e.key === 'Enter') {
						this.populateCarousel(i);
					}
				});
			}
		}

		// Click handlers for previous and next carousel buttons
		attachNavListeners() {
			const prev = getElement('.prev');
			const next = getElement('.next');

			prev.addEventListener('click', () => this.showPrevImg());
			next.addEventListener('click', () => this.showNextImg());

			prev.addEventListener('keydown', (e) => {
				if(e.key === 'Enter') {
					this.showPrevImg();
				}
			});

			next.addEventListener('keydown', (e) => {
				if(e.key === 'Enter') {
					this.showNextImg();
				}
			});
		}

		// Handle overlay when close icon or outside container is clicked
		attachOverlayListeners() {
			const carouselOverlay = getElement('.carousel-overlay');
			const carouselContainer = getElement('.carousel-container');
			const close = getElement('.close');

			// Close icon clicked
			close.addEventListener('click', () => {				
				this.closeOverlay();		
			});

			close.addEventListener('keydown', (e) => {
				if(e.key === 'Enter') {
					this.closeOverlay();
				}
			});

			// Overlay background outside of carousel container clicked
			carouselOverlay.addEventListener('click', () => {
				this.closeOverlay();
			});

			// Prevent close if carousel container itself is clicked
			carouselContainer.addEventListener('click', (e) => {
				e.stopPropagation();
			});
		}

		// Allow keyboard inputs to navigate carousel
		attachKeyboardListeners() {
			document.addEventListener('keydown', (e) => {
				switch(e.key) {
					case 'Escape':
						this.closeOverlay();
						break;
					case 'ArrowLeft':
						this.showPrevImg();
						break;
					case 'ArrowRight':
						this.showNextImg();
						break;
				}
			});
		}

		// Add touch event listeners on carousel
		attachTouchListeners() {
			const carousel = getElement('.carousel');
			this.initialX = null;
			this.initialY = null;
			this.eventFired = false;

			carousel.addEventListener('touchstart', (e) => this.touchStart(e));
			carousel.addEventListener('touchmove', (e) => this.touchMove(e));
		}

		// Store initial touch position
		touchStart(e) {
			this.initialX = e.touches[0].clientX;
			this.initialY = e.touches[0].clientY;
		}

		// Make call to updateImgFromTouch function to get swipe information
		touchMove(e) {
			if(this.eventFired === false) {
				this.updateImgFromTouch(e);
				this.eventFired = true;
				setTimeout(() => { // Throttle and only fire event once every 300ms
					this.eventFired = false;
				}, 300);
			}
		}

		// Get next or previous photo based on swipe gestures
		updateImgFromTouch(e) {
			// Prevent default browser behaviors (navigation, refresh, etc.)
			e.preventDefault();

			if(this.initialX === null || this.initialY === null) {
				return;
			}

			// Store current (ending) touch position
			const currX = e.touches[0].clientX;
			const currY = e.touches[0].clientY;

			// Calculate coordinate difference
			const diffX = currX - this.initialX;
			const diffY = currY - this.initialY;

			// Determine direction user swiped and show corresponding image
			if(Math.abs(diffX) > Math.abs(diffY)) {
				if(currX < this.initialX) {
					this.showNextImg();
				}
				else {
					this.showPrevImg();
				}
			}

			// Reset position to null to prevent constant firing events
			this.initialX = null;
			this.initialY = null;
		}

		// Add carousel overlay and content
		populateCarousel(currIndex) {
			const carouselOverlay = getElement('.carousel-overlay');
			const body = getElement('body');

			carouselOverlay.style.display = 'flex';
			body.classList.add('noscroll');
			this.setActiveImg(currIndex);
		}

		showPrevImg() {
			this.setActiveImg(this.getPrev(this.currIndex))
		}

		showNextImg() {
			this.setActiveImg(this.getNext(this.currIndex));
		}

		// Set active image and caption in carousel view
		setActiveImg(index) {
			const thumbnails = getElements('.photo-img');
			const caption = getElement('.caption');
			const carouselImg = document.querySelector('.carousel-img'); // Don't want to cache using getElement
			this.currIndex = index;

			// Clone carouselImg node so CSS fade animation can be reset
			const carouselImgClone = carouselImg.cloneNode(true);
			carouselImg.parentNode.replaceChild(carouselImgClone, carouselImg);

			caption.innerHTML = thumbnails[index].alt;
			carouselImgClone.src = thumbnails[index].src;
			carouselImgClone.alt = thumbnails[index].alt;
			carouselImgClone.classList.add('carousel-img-fade');
		}

		// Return index of previous image in carousel
		getPrev(currIndex) {
			if(currIndex !== 0) {
				currIndex--;
			}
			else {
				currIndex = imgArr.length-1;
			}
			return currIndex;
		}

		// Return index of next image in carousel
		getNext(currIndex) {
			if(currIndex !== imgArr.length-1) {
				currIndex++;
			}
			else {
				currIndex = 0;
			}
			return currIndex;
		}

		// Add carousel overlay and prevent page scrolling
		closeOverlay() {
			const carouselOverlay = getElement('.carousel-overlay');
			const body = getElement('body');

			carouselOverlay.style.display = 'none';
			body.classList.remove('noscroll');
		}
	}

	const imgGrid = new ImgGrid(imgArr);

})();